﻿Imports System.ComponentModel
Imports System.Globalization

Namespace Petzold.Controls
    Public Class DatePresenter
        Implements INotifyPropertyChanged
        Private Const MINYEAR As Integer = 1601
        Private Const MAXYEAR As Integer = 3000

        Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

        Private _selectedYear As YearInfo
        Private _selectedMonth As MonthInfo
        Private _selectedDay As DayInfo

        Public Sub New()
            AllYears = New YearInfo(MAXYEAR - MINYEAR){}

            For year As Integer = 0 To AllYears.Length - 1
                AllYears(year) = New YearInfo With {.YearNumber = year + MINYEAR}
            Next year

            AllMonths = New MonthInfo(11){}

            For month As Integer = 0 To 11
                AllMonths(month) = New MonthInfo With {.MonthNumber = month + 1}
            Next month

            _Date = Date.Now
        End Sub

        Private _AllYears As YearInfo()
        Public Property AllYears() As YearInfo()
            Get
                Return _AllYears
            End Get
            Protected Set(ByVal value As YearInfo())
                _AllYears = value
            End Set
        End Property

        Private _AllMonths As MonthInfo()
        Public Property AllMonths() As MonthInfo()
            Get
                Return _AllMonths
            End Get
            Protected Set(ByVal value As MonthInfo())
                _AllMonths = value
            End Set
        End Property

        Private _AllDays As DayInfo()
        Public Property AllDays() As DayInfo()
            Get
                Return _AllDays
            End Get
            Protected Set(ByVal value As DayInfo())
                _AllDays = value
            End Set
        End Property

        Public Property _Date() As Date
            Set(ByVal value As Date)
                If SelectedYear Is Nothing OrElse SelectedMonth Is Nothing OrElse SelectedDay Is Nothing OrElse value.Year <> SelectedYear.YearNumber OrElse value.Month <> SelectedMonth.MonthNumber OrElse value.Day <> SelectedDay.DayNumber Then
                    SelectedYear = AllYears(value.Year - MINYEAR)
                    SelectedMonth = AllMonths(value.Month - 1)
                    SelectedDay = AllDays(value.Day - 1)

                    OnPropertyChanged(New PropertyChangedEventArgs("DateTime"))
                End If
            End Set
            Get
                If SelectedYear IsNot Nothing AndAlso SelectedMonth IsNot Nothing AndAlso SelectedDay IsNot Nothing Then
                    Return New Date(SelectedYear.YearNumber, SelectedMonth.MonthNumber, SelectedDay.DayNumber)
                End If

                Return New Date()
            End Get
        End Property

        Public Property SelectedDay() As DayInfo
            Set(ByVal value As DayInfo)
                If value IsNot _selectedDay Then
                    _selectedDay = value
                    OnPropertyChanged(New PropertyChangedEventArgs("SelectedDay"))
                    OnPropertyChanged(New PropertyChangedEventArgs("DateTime"))
                End If
            End Set
            Get
                Return _selectedDay
            End Get
        End Property

        Public Property SelectedMonth() As MonthInfo
            Set(ByVal value As MonthInfo)
                If value IsNot _selectedMonth Then
                    _selectedMonth = value

                    If SelectedYear IsNot Nothing AndAlso value IsNot Nothing Then
                        MakeNewDays(SelectedYear.YearNumber, value.MonthNumber)
                    End If

                    OnPropertyChanged(New PropertyChangedEventArgs("SelectedMonth"))
                    OnPropertyChanged(New PropertyChangedEventArgs("DateTime"))
                End If
            End Set

            Get
                Return _selectedMonth
            End Get
        End Property

        Public Property SelectedYear() As YearInfo
            Set(ByVal value As YearInfo)
                If value IsNot _selectedYear Then
                    _selectedYear = value

                    If value IsNot Nothing AndAlso SelectedMonth IsNot Nothing Then
                        MakeNewDays(value.YearNumber, SelectedMonth.MonthNumber)
                    End If

                    OnPropertyChanged(New PropertyChangedEventArgs("SelectedYear"))
                    OnPropertyChanged(New PropertyChangedEventArgs("DateTime"))
                End If
            End Set
            Get
                Return _selectedYear
            End Get
        End Property

        Private Sub MakeNewDays(ByVal year As Integer, ByVal month As Integer)
            Dim daysInMonth As Integer = Date.DaysInMonth(year, month)
            AllDays = New DayInfo(daysInMonth - 1){}

            For day As Integer = 0 To daysInMonth - 1
                AllDays(day) = New DayInfo With {.Year = year, .Month = month, .DayNumber = day + 1}
            Next day

            OnPropertyChanged(New PropertyChangedEventArgs("AllDays"))

            If SelectedDay IsNot Nothing Then
                SelectedDay = AllDays(Math.Min(SelectedDay.DayNumber, AllDays.Length) - 1)
            End If
        End Sub

        Private Sub OnPropertyChanged(ByVal args As PropertyChangedEventArgs)
            RaiseEvent PropertyChanged(Me, args)
        End Sub
    End Class
End Namespace
