﻿Imports System.ComponentModel

Namespace Petzold.Controls
    Partial Public Class WindowedItemsControl
        Private Const SLIDE_TIME As Integer = 250 ' milliseconds
        Private Const DECELERATION As Double = 0.001 ' units per msec squared

        Private manipulationInProgress As Boolean
        Private manipulationDeltaInProgress As Boolean
        Private tapInertiaInProgess As Boolean
        Private inertiaToUnknownIndex As Boolean
        Private reverseInertiaChecked As Boolean
        Private inertiaDirection As Integer

        Protected Overrides Sub OnManipulationStarting(ByVal args As ManipulationStartingEventArgs)
            args.ManipulationContainer = Me
            args.Mode = ManipulationModes.TranslateY

            ' Initialize all fields
            manipulationInProgress = True
            manipulationDeltaInProgress = False
            tapInertiaInProgess = False
            inertiaToUnknownIndex = False
            reverseInertiaChecked = False
            inertiaDirection = 1

            args.Handled = True
            MyBase.OnManipulationStarting(args)
        End Sub

        Protected Overrides Sub OnManipulationDelta(ByVal args As ManipulationDeltaEventArgs)
            If Not manipulationDeltaInProgress Then
                IsActive = True

                If Not tapInertiaInProgess Then
                    SelectedIndex = -1
                End If
            End If

            manipulationDeltaInProgress = True

            ' This is the fake inertia from a tap
            If tapInertiaInProgess Then
                VerticalOffset -= args.DeltaManipulation.Translation.Y

            ' All other direct manipulation and inertia
            Else
                ' For non-wrappable panel, check for end of the line
                If Not isWrappableStackPanel Then
                    Dim newVerticalOffset As Double = VerticalOffset - inertiaDirection * args.DeltaManipulation.Translation.Y

                    If FractionalCenteredIndexFromVerticalOffset(newVerticalOffset, False) < 0 Then
                        Dim verticalOffsetIncrement As Double = VerticalOffset - VerticalOffsetFromCenteredIndex(0)
                        Dim verticalOffsetExcess As Double = args.DeltaManipulation.Translation.Y - verticalOffsetIncrement
                        VerticalOffset -= verticalOffsetIncrement
                        SelectedIndex = 0
                        args.ReportBoundaryFeedback(New ManipulationDelta(New Vector(0, verticalOffsetExcess), 0, New Vector(), New Vector()))
                        args.Complete()
                    ElseIf FractionalCenteredIndexFromVerticalOffset(newVerticalOffset, False) > Items.Count - 1 Then
                        Dim verticalOffsetIncrement As Double = VerticalOffsetFromCenteredIndex(Me.Items.Count - 1) - VerticalOffset
                        Dim verticalOffsetExcess As Double = args.DeltaManipulation.Translation.Y - verticalOffsetIncrement
                        VerticalOffset += verticalOffsetIncrement
                        SelectedIndex = Items.Count - 1
                        args.ReportBoundaryFeedback(New ManipulationDelta(New Vector(0, verticalOffsetExcess), 0, New Vector(), New Vector()))
                        args.Complete()
                    End If
                End If

                ' Here's where scrolling might reverse itself
                If args.IsInertial AndAlso inertiaToUnknownIndex AndAlso (Not reverseInertiaChecked) Then
                    CheckForBackupManeuver(VerticalOffset, args.DeltaManipulation.Translation.Y, args.Velocities.LinearVelocity.Y)
                End If

                ' This is the normal direct manipulation and inertia
                VerticalOffset -= inertiaDirection * args.DeltaManipulation.Translation.Y
            End If

            MyBase.OnManipulationDelta(args)
        End Sub


        Private Sub CheckForBackupManeuver(ByVal verticalOffset As Double, ByVal translation As Double, ByVal velocity As Double)
            Dim newVerticalOffset As Double = verticalOffset - inertiaDirection * translation
            Dim stopDistance As Double = velocity * velocity / (2 * DECELERATION)
            Dim index As Double = FractionalCenteredIndexFromVerticalOffset(verticalOffset, False)
            Dim prevIndex, nextIndex As Integer

            If inertiaDirection * translation > 0 Then
                prevIndex = CInt(Fix(Math.Ceiling(index)))
                nextIndex = CInt(Fix(Math.Floor(index)))
            Else
                prevIndex = CInt(Fix(Math.Floor(index)))
                nextIndex = CInt(Fix(Math.Ceiling(index)))
            End If

            Dim prevVerticalOffset As Double = VerticalOffsetFromCenteredIndex(prevIndex)
            Dim nextVerticalOffset As Double = VerticalOffsetFromCenteredIndex(nextIndex)

            If stopDistance < Math.Abs(newVerticalOffset - nextVerticalOffset) AndAlso stopDistance < Math.Abs(newVerticalOffset - prevVerticalOffset) Then
                reverseInertiaChecked = True

                If Math.Abs(newVerticalOffset - prevVerticalOffset) < Math.Abs(newVerticalOffset - nextVerticalOffset) Then
                    inertiaDirection = -inertiaDirection
                    SelectedIndex = NormalizeIndex(prevIndex)
                Else
                    SelectedIndex = NormalizeIndex(nextIndex)
                End If
            End If
        End Sub

        Protected Overrides Sub OnManipulationInertiaStarting(ByVal args As ManipulationInertiaStartingEventArgs)
            ' Check to see if there have been any deltas; if not, it's a tap
            If Not manipulationDeltaInProgress Then
                ' This might be negative or beyond bounds!
                Dim tappedItemIndex As Integer = CInt(Fix(Math.Floor((args.ManipulationOrigin.Y + VerticalOffset) / ItemsHeight)))
                Dim destinationVerticalOffset As Double = VerticalOffsetFromCenteredIndex(tappedItemIndex)
                Dim displacement As Double = VerticalOffset - destinationVerticalOffset

                ' Set SelectedIndex
                SelectedIndex = NormalizeIndex(tappedItemIndex)

                ' Now start inertia to slide into place
                If displacement <> 0 Then
                    Dim velocity As Double = 2 * displacement / SLIDE_TIME
                    args.TranslationBehavior.DesiredDisplacement = Math.Abs(displacement)
                    args.TranslationBehavior.InitialVelocity = New Vector(0, velocity)
                    tapInertiaInProgess = True
                End If
            ' Not a tap -- there has been some movement and now the finger has lifted
            Else
                ' Check for insufficient velocity
                Dim closestIndex As Integer = CInt(Fix(Math.Round(FractionalCenteredIndexFromVerticalOffset(VerticalOffset, False))))
                Dim VerticalOffsetOfClosestIndex As Double = VerticalOffsetFromCenteredIndex(closestIndex)
                Dim displacement As Double = VerticalOffset - VerticalOffsetOfClosestIndex
                Dim velocity As Double = 2 * displacement / SLIDE_TIME

                ' Band has been slid precisely into place
                If displacement = 0 Then
                    SelectedIndex = NormalizeIndex(closestIndex)
                ' Insufficient velocity to move beyond "destinationIndex"
                ElseIf Math.Abs(args.TranslationBehavior.InitialVelocity.Y) < Math.Abs(velocity) Then
                    SelectedIndex = NormalizeIndex(closestIndex)
                    args.TranslationBehavior.DesiredDisplacement = Math.Abs(displacement)
                    args.TranslationBehavior.InitialVelocity = New Vector(0, velocity)
                ' Enough velocity for inertia to some unknown index
                Else
                    inertiaToUnknownIndex = True
                    args.TranslationBehavior.DesiredDeceleration = DECELERATION
                End If
            End If

            args.Handled = True
            MyBase.OnManipulationInertiaStarting(args)
        End Sub

        Protected Overrides Sub OnManipulationCompleted(ByVal args As ManipulationCompletedEventArgs)
            If Not manipulationDeltaInProgress Then
                IsActive = IsActive Xor True
            End If

            manipulationInProgress = False
            SetVerticalOffsetFromSelectedIndex()

            args.Handled = True
            MyBase.OnManipulationCompleted(args)
        End Sub
    End Class
End Namespace
