﻿Imports System.ComponentModel
Imports System.Windows.Controls.Primitives

Namespace Petzold.Controls
    Partial Public Class WindowedItemsControl
        Inherits ItemsControl
        Private isWrappableStackPanel As Boolean
        Private numberItems As Integer
        Private ItemsHeight As Double
        Private ViewportHeight As Double
        Private ExtentHeight As Double

        Public Event IsActiveChanged As DependencyPropertyChangedEventHandler

        Public Shared ReadOnly IsActiveProperty As DependencyProperty = DependencyProperty.Register("IsActive", GetType(Boolean), GetType(WindowedItemsControl), New PropertyMetadata(False, AddressOf OnIsActiveChanged))

        Public Shared ReadOnly SelectedIndexProperty As DependencyProperty = Selector.SelectedIndexProperty.AddOwner(GetType(WindowedItemsControl), New FrameworkPropertyMetadata(-1, AddressOf OnSelectedIndexChanged))

        Public Shared ReadOnly SelectedItemProperty As DependencyProperty = Selector.SelectedItemProperty.AddOwner(GetType(WindowedItemsControl), New FrameworkPropertyMetadata(Nothing, AddressOf OnSelectedItemChanged))

        Private Shared ReadOnly VerticalOffsetPropertyKey As DependencyPropertyKey = DependencyProperty.RegisterReadOnly("VerticalOffset", GetType(Double), GetType(WindowedItemsControl), New PropertyMetadata(AddressOf OnVerticalOffsetChanged))

        Public Shared ReadOnly VerticalOffsetProperty As DependencyProperty = VerticalOffsetPropertyKey.DependencyProperty

        Public Shared ReadOnly StartIndexProperty As DependencyProperty = DependencyProperty.RegisterAttached("StartIndex", GetType(Double), GetType(WindowedItemsControl), New FrameworkPropertyMetadata(0.0, FrameworkPropertyMetadataOptions.Inherits))

        Public Sub New()
            IsManipulationEnabled = True
        End Sub

        Public Property IsActive() As Boolean
            Set(ByVal value As Boolean)
                SetValue(IsActiveProperty, value)
            End Set
            Get
                Return CBool(GetValue(IsActiveProperty))
            End Get
        End Property

        Public Property SelectedIndex() As Integer
            Set(ByVal value As Integer)
                SetValue(SelectedIndexProperty, value)
            End Set
            Get
                Return CInt(Fix(GetValue(SelectedIndexProperty)))
            End Get
        End Property

        Public Property SelectedItem() As Object
            Set(ByVal value As Object)
                SetValue(SelectedItemProperty, value)
            End Set
            Get
                Return CObj(GetValue(SelectedItemProperty))
            End Get
        End Property

        Public Property VerticalOffset() As Double
            Protected Set(ByVal value As Double)
                SetValue(VerticalOffsetPropertyKey, value)
            End Set
            Get
                Return CDbl(GetValue(VerticalOffsetProperty))
            End Get
        End Property

        Public Property StartIndex() As Double
            Set(ByVal value As Double)
                SetValue(StartIndexProperty, value)
            End Set
            Get
                Return CDbl(GetValue(StartIndexProperty))
            End Get
        End Property

        ' IsActive property-changed handlers
        Private Shared Sub OnIsActiveChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
            TryCast(obj, WindowedItemsControl).OnIsActiveChanged(args)
        End Sub

        Private Sub OnIsActiveChanged(ByVal args As DependencyPropertyChangedEventArgs)
            RaiseEvent IsActiveChanged(Me, args)
        End Sub

        ' SelectedIndex property-changed handlers
        Private Shared Sub OnSelectedIndexChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
            TryCast(obj, WindowedItemsControl).OnSelectedIndexChanged(args)
        End Sub

        Private Sub OnSelectedIndexChanged(ByVal args As DependencyPropertyChangedEventArgs)
            If SelectedIndex >= 0 AndAlso SelectedIndex < Me.Items.Count Then
                SelectedItem = Me.Items(SelectedIndex)
            Else
                SelectedItem = Nothing
            End If

            SetVerticalOffsetFromSelectedIndex()
        End Sub

        ' SelectedItem property-changed handlers
        Private Shared Sub OnSelectedItemChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
            TryCast(obj, WindowedItemsControl).OnSelectedItemChanged(args)
        End Sub

        Private Sub OnSelectedItemChanged(ByVal args As DependencyPropertyChangedEventArgs)
            If SelectedItem Is Nothing Then
                SelectedIndex = -1
            Else
                SelectedIndex = Me.Items.IndexOf(SelectedItem)
            End If
        End Sub

        Private Shared Sub OnVerticalOffsetChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
            TryCast(obj, WindowedItemsControl).OnVerticalOffsetChanged(args)
        End Sub

        Private Sub OnVerticalOffsetChanged(ByVal args As DependencyPropertyChangedEventArgs)
            If Not isWrappableStackPanel Then
                InvalidateArrange()
            ElseIf ItemsHeight > 0 AndAlso numberItems > 0 Then
                Dim startIndex As Double = VerticalOffset / ItemsHeight

                startIndex = startIndex Mod numberItems
                startIndex = (startIndex + numberItems) Mod numberItems
                Me.StartIndex = startIndex
            End If
        End Sub

        Private Sub SetVerticalOffsetFromSelectedIndex()
            numberItems = Me.Items.Count
            ItemsHeight = If(numberItems > 0, ExtentHeight / numberItems, 0)

            If SelectedIndex <> -1 AndAlso (Not manipulationInProgress) Then
                VerticalOffset = VerticalOffsetFromCenteredIndex(SelectedIndex)
            End If
        End Sub

        Private Function VerticalOffsetFromCenteredIndex(ByVal index As Integer) As Double
            Return index * ItemsHeight - (ViewportHeight - ItemsHeight) / 2
        End Function

        Private Function NormalizeIndex(ByVal index As Integer) As Integer
            If numberItems = 0 Then
                Return 0
            End If

            Do While index < 0
                index += numberItems
            Loop

            Do While index >= numberItems
                index -= numberItems
            Loop

            Return index
        End Function

        ' not used yet
        Private Function NormalizeIndex(ByVal index As Double) As Double
            If numberItems = 0 Then
                Return 0
            End If

            Do While index < 0
                index += numberItems
            Loop

            Do While index >= numberItems
                index -= numberItems
            Loop

            Return index
        End Function

        Private Function FractionalCenteredIndexFromVerticalOffset(ByVal verticalOffset As Double, ByVal normalize As Boolean) As Double
            Dim index As Double = (verticalOffset + (ViewportHeight - ItemsHeight) / 2) / ItemsHeight

            If normalize Then
                index = index Mod numberItems
                index = (index + numberItems) Mod numberItems
            End If

            Return index
        End Function
    End Class
End Namespace
