﻿Namespace Petzold.Controls
    Public Class WrappableStackPanel
        Inherits Panel
        Private itemsHeight As Double

        Public Shared ReadOnly StartIndexProperty As DependencyProperty = WindowedItemsControl.StartIndexProperty.AddOwner(GetType(WrappableStackPanel), New FrameworkPropertyMetadata(0.0, FrameworkPropertyMetadataOptions.AffectsArrange))

        Public Property StartIndex() As Double
            Set(ByVal value As Double)
                SetValue(StartIndexProperty, value)
            End Set
            Get
                Return CDbl(GetValue(StartIndexProperty))
            End Get
        End Property

        Protected Overrides Function MeasureOverride(ByVal availableSize As Size) As Size
            Dim compositeSize As New Size()

            For Each child As UIElement In InternalChildren
                child.Measure(New Size(availableSize.Width, Double.PositiveInfinity))
                compositeSize.Width = Math.Max(compositeSize.Width, child.DesiredSize.Width)
                compositeSize.Height += child.DesiredSize.Height
            Next child

            If InternalChildren.Count = 0 Then
                itemsHeight = 0
            Else

                itemsHeight = compositeSize.Height / InternalChildren.Count
            End If

            Return compositeSize
        End Function

        Protected Overrides Function ArrangeOverride(ByVal finalSize As Size) As Size
            If InternalChildren.Count = 0 Then
                Return MyBase.ArrangeOverride(finalSize)
            End If

            Dim count As Integer = InternalChildren.Count
            Dim intStartIndex As Integer = CInt(Fix(StartIndex))
            Dim y As Double = -(StartIndex Mod 1) * itemsHeight

            For index As Integer = 0 To count - 1
                Dim child As UIElement = InternalChildren((intStartIndex + index) Mod count)
                child.Arrange(New Rect(New Point(0, y), New Size(finalSize.Width, child.DesiredSize.Height)))
                y += child.DesiredSize.Height
            Next index

            Return MyBase.ArrangeOverride(finalSize)
        End Function
    End Class
End Namespace
